@@ -1,6 +1,6 @@
 /*
  * sar, sadc, sadf, mpstat and iostat common routines.
- * (C) 1999-2017 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2018 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -35,8 +35,6 @@
 
 #include "version.h"
 #include "common.h"
-#include "ioconf.h"
-#include "rd_stats.h"
 
 #ifdef USE_NLS
 #include <locale.h>
@@ -46,11 +44,14 @@
 #define _(string) (string)
 #endif
 
+/* Number of decimal places */
+extern int dplaces_nr;
+
 /* Units (sectors, Bytes, kilobytes, etc.) */
 char units[] = {'s', 'B', 'k', 'M', 'G', 'T', 'P', '?'};
 
 /* Number of ticks per second */
-unsigned int hz;
+unsigned long hz;
 /* Number of bit shifts to convert pages to kB */
 unsigned int kb_shift;
 
@@ -170,6 +171,113 @@ time_t get_time(struct tm *rectime, int d_off)
 		return get_localtime(rectime, d_off);
 }
 
+#ifdef USE_NLS
+/*
+ ***************************************************************************
+ * Init National Language Support.
+ ***************************************************************************
+ */
+void init_nls(void)
+{
+	setlocale(LC_MESSAGES, "");
+	setlocale(LC_CTYPE, "");
+	setlocale(LC_TIME, "");
+	setlocale(LC_NUMERIC, "");
+
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	textdomain(PACKAGE);
+}
+#endif
+
+/*
+ ***************************************************************************
+ * Test whether given name is a device or a partition, using sysfs.
+ * This is more straightforward that using ioc_iswhole() function from
+ * ioconf.c which should be used only with kernels that don't have sysfs.
+ *
+ * IN:
+ * @name		Device or partition name.
+ * @allow_virtual	TRUE if virtual devices are also accepted.
+ *			The device is assumed to be virtual if no
+ *			/sys/block/<device>/device link exists.
+ *
+ * RETURNS:
+ * TRUE if @name is not a partition.
+ ***************************************************************************
+ */
+int is_device(char *name, int allow_virtual)
+{
+	char syspath[PATH_MAX];
+	char *slash;
+
+	/* Some devices may have a slash in their name (eg. cciss/c0d0...) */
+	while ((slash = strchr(name, '/'))) {
+		*slash = '!';
+	}
+	snprintf(syspath, sizeof(syspath), "%s/%s%s", SYSFS_BLOCK, name,
+		 allow_virtual ? "" : "/device");
+
+	return !(access(syspath, F_OK));
+}
+
+/*
+ ***************************************************************************
+ * Get page shift in kB.
+ ***************************************************************************
+ */
+void get_kb_shift(void)
+{
+	int shift = 0;
+	long size;
+
+	/* One can also use getpagesize() to get the size of a page */
+	if ((size = sysconf(_SC_PAGESIZE)) == -1) {
+		perror("sysconf");
+	}
+
+	size >>= 10;	/* Assume that a page has a minimum size of 1 kB */
+
+	while (size > 1) {
+		shift++;
+		size >>= 1;
+	}
+
+	kb_shift = (unsigned int) shift;
+}
+
+/*
+ ***************************************************************************
+ * Get number of clock ticks per second.
+ ***************************************************************************
+ */
+void get_HZ(void)
+{
+	long ticks;
+
+	if ((ticks = sysconf(_SC_CLK_TCK)) == -1) {
+		perror("sysconf");
+	}
+
+	hz = (unsigned long) ticks;
+}
+
+/*
+ ***************************************************************************
+ * Unhandled situation: Panic and exit. Should never happen.
+ *
+ * IN:
+ * @function	Function name where situation occured.
+ * @error_code	Error code.
+ ***************************************************************************
+ */
+void sysstat_panic(const char *function, int error_code)
+{
+	fprintf(stderr, "sysstat: %s[%d]: Internal error...\n",
+		function, error_code);
+	exit(1);
+}
+
+#ifndef SOURCE_SADC
 /*
  ***************************************************************************
  * Count number of comma-separated values in arguments list. For example,
@@ -217,7 +325,7 @@ int get_dev_part_nr(char *dev_name)
 	DIR *dir;
 	struct dirent *drd;
 	char dfile[MAX_PF_NAME], line[MAX_PF_NAME];
-	int part = 0;
+	int part = 0, err;
 
 	snprintf(dfile, MAX_PF_NAME, "%s/%s", SYSFS_BLOCK, dev_name);
 	dfile[MAX_PF_NAME - 1] = '\0';
@@ -230,8 +338,9 @@ int get_dev_part_nr(char *dev_name)
 	while ((drd = readdir(dir)) != NULL) {
 		if (!strcmp(drd->d_name, ".") || !strcmp(drd->d_name, ".."))
 			continue;
-		snprintf(line, MAX_PF_NAME, "%s/%s/%s", dfile, drd->d_name, S_STAT);
-		line[MAX_PF_NAME - 1] = '\0';
+		err = snprintf(line, MAX_PF_NAME, "%s/%s/%s", dfile, drd->d_name, S_STAT);
+		if ((err < 0) || (err >= MAX_PF_NAME))
+			continue;
 
 		/* Try to guess if current entry is a directory containing a stat file */
 		if (!access(line, R_OK)) {
@@ -495,24 +604,6 @@ int print_gal_header(struct tm *rectime, char *sysname, char *release,
 	return rc;
 }
 
-#ifdef USE_NLS
-/*
- ***************************************************************************
- * Init National Language Support.
- ***************************************************************************
- */
-void init_nls(void)
-{
-	setlocale(LC_MESSAGES, "");
-	setlocale(LC_CTYPE, "");
-	setlocale(LC_TIME, "");
-	setlocale(LC_NUMERIC, "");
-
-	bindtextdomain(PACKAGE, LOCALEDIR);
-	textdomain(PACKAGE);
-}
-#endif
-
 /*
  ***************************************************************************
  * Get number of rows for current window.
@@ -574,78 +665,6 @@ char *device_name(char *name)
 	return out;
 }
 
-/*
- ***************************************************************************
- * Test whether given name is a device or a partition, using sysfs.
- * This is more straightforward that using ioc_iswhole() function from
- * ioconf.c which should be used only with kernels that don't have sysfs.
- *
- * IN:
- * @name		Device or partition name.
- * @allow_virtual	TRUE if virtual devices are also accepted.
- *			The device is assumed to be virtual if no
- *			/sys/block/<device>/device link exists.
- *
- * RETURNS:
- * TRUE if @name is not a partition.
- ***************************************************************************
- */
-int is_device(char *name, int allow_virtual)
-{
-	char syspath[PATH_MAX];
-	char *slash;
-
-	/* Some devices may have a slash in their name (eg. cciss/c0d0...) */
-	while ((slash = strchr(name, '/'))) {
-		*slash = '!';
-	}
-	snprintf(syspath, sizeof(syspath), "%s/%s%s", SYSFS_BLOCK, name,
-		 allow_virtual ? "" : "/device");
-
-	return !(access(syspath, F_OK));
-}
-
-/*
- ***************************************************************************
- * Get page shift in kB.
- ***************************************************************************
- */
-void get_kb_shift(void)
-{
-	int shift = 0;
-	long size;
-
-	/* One can also use getpagesize() to get the size of a page */
-	if ((size = sysconf(_SC_PAGESIZE)) == -1) {
-		perror("sysconf");
-	}
-
-	size >>= 10;	/* Assume that a page has a minimum size of 1 kB */
-
-	while (size > 1) {
-		shift++;
-		size >>= 1;
-	}
-
-	kb_shift = (unsigned int) shift;
-}
-
-/*
- ***************************************************************************
- * Get number of clock ticks per second.
- ***************************************************************************
- */
-void get_HZ(void)
-{
-	long ticks;
-
-	if ((ticks = sysconf(_SC_CLK_TCK)) == -1) {
-		perror("sysconf");
-	}
-
-	hz = (unsigned int) ticks;
-}
-
 /*
  ***************************************************************************
  * Workaround for CPU counters read from /proc/stat: Dyn-tick kernels
@@ -666,11 +685,11 @@ double ll_sp_value(unsigned long long value1, unsigned long long value2,
  * Compute time interval.
  *
  * IN:
- * @prev_uptime	Previous uptime value in jiffies.
- * @curr_uptime	Current uptime value in jiffies.
+ * @prev_uptime	Previous uptime value (in jiffies or 1/100th of a second).
+ * @curr_uptime	Current uptime value (in jiffies or 1/100th of a second).
  *
  * RETURNS:
- * Interval of time in jiffies.
+ * Interval of time in jiffies or 1/100th of a second.
  ***************************************************************************
  */
 unsigned long long get_interval(unsigned long long prev_uptime,
@@ -688,93 +707,6 @@ unsigned long long get_interval(unsigned long long prev_uptime,
 	return itv;
 }
 
-/*
- ***************************************************************************
- * Since ticks may vary slightly from CPU to CPU, we'll want
- * to recalculate itv based on this CPU's tick count, rather
- * than that reported by the "cpu" line. Otherwise we
- * occasionally end up with slightly skewed figures, with
- * the skew being greater as the time interval grows shorter.
- *
- * IN:
- * @scc	Current sample statistics for current CPU.
- * @scp	Previous sample statistics for current CPU.
- *
- * RETURNS:
- * Interval of time based on current CPU.
- ***************************************************************************
- */
-unsigned long long get_per_cpu_interval(struct stats_cpu *scc,
-					struct stats_cpu *scp)
-{
-	unsigned long long ishift = 0LL;
-
-	if ((scc->cpu_user - scc->cpu_guest) < (scp->cpu_user - scp->cpu_guest)) {
-		/*
-		 * Sometimes the nr of jiffies spent in guest mode given by the guest
-		 * counter in /proc/stat is slightly higher than that included in
-		 * the user counter. Update the interval value accordingly.
-		 */
-		ishift += (scp->cpu_user - scp->cpu_guest) -
-		          (scc->cpu_user - scc->cpu_guest);
-	}
-	if ((scc->cpu_nice - scc->cpu_guest_nice) < (scp->cpu_nice - scp->cpu_guest_nice)) {
-		/*
-		 * Idem for nr of jiffies spent in guest_nice mode.
-		 */
-		ishift += (scp->cpu_nice - scp->cpu_guest_nice) -
-		          (scc->cpu_nice - scc->cpu_guest_nice);
-	}
-
-	/*
-	 * Workaround for CPU coming back online: With recent kernels
-	 * some fields (user, nice, system) restart from their previous value,
-	 * whereas others (idle, iowait) restart from zero.
-	 * For the latter we need to set their previous value to zero to
-	 * avoid getting an interval value < 0.
-	 * (I don't know how the other fields like hardirq, steal... behave).
-	 * Don't assume the CPU has come back from offline state if previous
-	 * value was greater than ULLONG_MAX & 0x7ffff (the counter probably
-	 * overflew).
-	 */
-	if ((scc->cpu_idle < scp->cpu_idle) && (scp->cpu_idle < (ULLONG_MAX - 0x7ffff))) {
-		scp->cpu_idle = 0;
-	}
-	if ((scc->cpu_iowait < scp->cpu_iowait) && (scp->cpu_iowait < (ULLONG_MAX - 0x7ffff))) {
-		scp->cpu_iowait = 0;
-	}
-
-	/*
-	 * Don't take cpu_guest and cpu_guest_nice into account
-	 * because cpu_user and cpu_nice already include them.
-	 */
-	return ((scc->cpu_user    + scc->cpu_nice   +
-		 scc->cpu_sys     + scc->cpu_iowait +
-		 scc->cpu_idle    + scc->cpu_steal  +
-		 scc->cpu_hardirq + scc->cpu_softirq) -
-		(scp->cpu_user    + scp->cpu_nice   +
-		 scp->cpu_sys     + scp->cpu_iowait +
-		 scp->cpu_idle    + scp->cpu_steal  +
-		 scp->cpu_hardirq + scp->cpu_softirq) +
-		 ishift);
-}
-
-/*
- ***************************************************************************
- * Unhandled situation: Panic and exit. Should never happen.
- *
- * IN:
- * @function	Function name where situation occured.
- * @error_code	Error code.
- ***************************************************************************
- */
-void sysstat_panic(const char *function, int error_code)
-{
-	fprintf(stderr, "sysstat: %s[%d]: Internal error...\n",
-		function, error_code);
-	exit(1);
-}
-
 /*
  ***************************************************************************
  * Count number of bits set in an array.
@@ -805,39 +737,6 @@ int count_bits(void *ptr, int size)
 	return nr;
 }
 
-/*
- ***************************************************************************
- * Compute "extended" device statistics (service time, etc.).
- *
- * IN:
- * @sdc		Structure with current device statistics.
- * @sdp		Structure with previous device statistics.
- * @itv		Interval of time in jiffies.
- *
- * OUT:
- * @xds		Structure with extended statistics.
- ***************************************************************************
-*/
-void compute_ext_disk_stats(struct stats_disk *sdc, struct stats_disk *sdp,
-			    unsigned long long itv, struct ext_disk_stats *xds)
-{
-	double tput
-		= ((double) (sdc->nr_ios - sdp->nr_ios)) * HZ / itv;
-
-	xds->util  = S_VALUE(sdp->tot_ticks, sdc->tot_ticks, itv);
-	xds->svctm = tput ? xds->util / tput : 0.0;
-	/*
-	 * Kernel gives ticks already in milliseconds for all platforms
-	 * => no need for further scaling.
-	 */
-	xds->await = (sdc->nr_ios - sdp->nr_ios) ?
-		((sdc->rd_ticks - sdp->rd_ticks) + (sdc->wr_ticks - sdp->wr_ticks)) /
-		((double) (sdc->nr_ios - sdp->nr_ios)) : 0.0;
-	xds->arqsz = (sdc->nr_ios - sdp->nr_ios) ?
-		((sdc->rd_sect - sdp->rd_sect) + (sdc->wr_sect - sdp->wr_sect)) /
-		((double) (sdc->nr_ios - sdp->nr_ios)) : 0.0;
-}
-
 /*
  ***************************************************************************
  * Convert in-place input string to lowercase.
@@ -1172,7 +1071,7 @@ void cprintf_unit(int unit, int wi, double dval)
 		dval /= 1024;
 		unit++;
 	}
-	printf(" %*.*f", wi - 1, 1, dval);
+	printf(" %*.*f", wi - 1, dplaces_nr ? 1 : 0, dval);
 	printf("%s", sc_normal);
 
 	/* Display unit */
@@ -1262,15 +1161,28 @@ void cprintf_x(int num, int wi, ...)
 void cprintf_f(int unit, int num, int wi, int wd, ...)
 {
 	int i;
-	double val;
+	double val, lim = 0.005;;
 	va_list args;
 
+	/*
+	 * If there are decimal places then get the value
+	 * entered on the command line (if existing).
+	 */
+	if ((wd > 0) && (dplaces_nr >= 0)) {
+		wd = dplaces_nr;
+	}
+
+	/* Update limit value according to number of decimal places */
+	if (wd == 1) {
+		lim = 0.05;
+	}
+
 	va_start(args, wd);
 
 	for (i = 0; i < num; i++) {
 		val = va_arg(args, double);
-		if (((val < 0.005) && (val > -0.005)) ||
-		    ((wd == 0) && (val < 0.5))) {
+		if (((wd > 0) && (val < lim) && (val > (lim * -1))) ||
+		    ((wd == 0) && (val <= 0.5) && (val >= -0.5))) {	/* "Round half to even" law */
 			printf("%s", sc_zero_int_stat);
 		}
 		else {
@@ -1308,22 +1220,34 @@ void cprintf_pc(int human, int num, int wi, int wd, ...)
 	va_list args;
 
 	/*
-	 * If a percent sign is to be displayed, then there will be only one decimal place.
-	 * In this case, a value smaller than 0.05 shall be considered as 0.
+	 * If there are decimal places then get the value
+	 * entered on the command line (if existing).
+	 */
+	if ((wd > 0) && (dplaces_nr >= 0)) {
+		wd = dplaces_nr;
+	}
+
+	/*
+	 * If a percent sign is to be displayed, then there will be
+	 * zero (or one) decimal place.
 	 */
 	if (human > 0) {
-		lim = 0.05;
 		if (wi < 4) {
 			/* E.g., 100% */
 			wi = 4;
 		}
 		/* Keep one place for the percent sign */
 		wi -= 1;
-		if (wd > 0) {
+		if (wd > 1) {
 			wd -= 1;
 		}
 	}
 
+	/* Update limit value according to number of decimal places */
+	if (wd == 1) {
+		lim = 0.05;
+	}
+
 	va_start(args, wd);
 
 	for (i = 0; i < num; i++) {
@@ -1334,7 +1258,8 @@ void cprintf_pc(int human, int num, int wi, int wd, ...)
 		else if (val >= PERCENT_LIMIT_LOW) {
 			printf("%s", sc_percent_low);
 		}
-		else if (val < lim) {
+		else if (((wd > 0) && (val < lim)) ||
+			 ((wd == 0) && (val <= 0.5))) {	/* "Round half to even" law */
 			printf("%s", sc_zero_int_stat);
 		}
 		else {
@@ -1509,3 +1434,4 @@ int parse_values(char *strargv, unsigned char bitmap[], int max_val, const char
 
 	return 0;
 }
+#endif /* SOURCE_SADC undefined */
