@@ -1,7 +1,7 @@
 /*
  * count.h: Include file used to count items for which
  * statistics will be collected.
- * (C) 1999-2017 by Sebastien Godard (sysstat <at> orange.fr)
+ * (C) 1999-2018 by Sebastien Godard (sysstat <at> orange.fr)
  */
 
 #ifndef _COUNT_H
@@ -15,27 +15,27 @@
  ***************************************************************************
  */
 
-int get_cpu_nr
+__nr_t get_cpu_nr
 	(unsigned int, int);
-int get_irqcpu_nr
+__nr_t get_irqcpu_nr
 	(char *, int, int);
-int get_diskstats_dev_nr
+__nr_t get_diskstats_dev_nr
 	(int, int);
-int get_irq_nr
+__nr_t get_irq_nr
 	(void);
-int get_serial_nr
+__nr_t get_serial_nr
 	(void);
-int get_iface_nr
+__nr_t get_iface_nr
 	(void);
-int get_disk_nr
+__nr_t get_disk_nr
 	(unsigned int);
-int get_freq_nr
+__nr_t get_freq_nr
 	(void);
-int get_usb_nr
+__nr_t get_usb_nr
 	(void);
-int get_filesystem_nr
+__nr_t get_filesystem_nr
 	(void);
-int get_fchost_nr
+__nr_t get_fchost_nr
 	(void);
 
 #endif /* _COUNT_H */
