@@ -1,6 +1,6 @@
 /*
  * sadf: System activity data formatter
- * (C) 1999-2017 by Sebastien Godard (sysstat <at> orange.fr)
+ * (C) 1999-2018 by Sebastien Godard (sysstat <at> orange.fr)
  */
 
 #ifndef _SADF_H
@@ -17,7 +17,7 @@
 
 #define IGNORE_NOTHING		0
 #define IGNORE_RESTART		1
-#define DONT_READ_VOLATILE	2
+#define DONT_READ_CPU_NR	2
 #define IGNORE_COMMENT		4
 #define SET_TIMESTAMPS		8
 
@@ -130,17 +130,15 @@ void convert_file
  * Prototypes used to display restart messages
  */
 __printf_funct_t print_db_restart
-	(int *, int, char *, char *, int, struct file_header *, unsigned int);
+	(int *, int, char *, char *, int, struct file_header *);
 __printf_funct_t print_ppc_restart
-	(int *, int, char *, char *, int, struct file_header *, unsigned int);
+	(int *, int, char *, char *, int, struct file_header *);
 __printf_funct_t print_xml_restart
-	(int *, int, char *, char *, int, struct file_header *, unsigned int);
+	(int *, int, char *, char *, int, struct file_header *);
 __printf_funct_t print_json_restart
-	(int *, int, char *, char *, int, struct file_header *, unsigned int);
-__printf_funct_t print_sar_restart
-	(int *, int, char *, char *, int, struct file_header *, unsigned int);
+	(int *, int, char *, char *, int, struct file_header *);
 __printf_funct_t print_raw_restart
-	(int *, int, char *, char *, int, struct file_header *, unsigned int);
+	(int *, int, char *, char *, int, struct file_header *);
 
 /*
  * Prototypes used to display comments
@@ -185,15 +183,15 @@ __tm_funct_t print_raw_timestamp
  */
 __printf_funct_t print_xml_header
 	(void *, int, char *, struct file_magic *, struct file_header *,
-	 __nr_t, struct activity * [], unsigned int []);
+	 struct activity * [], unsigned int [], struct file_activity *);
 __printf_funct_t print_json_header
 	(void *, int, char *, struct file_magic *, struct file_header *,
-	 __nr_t, struct activity * [], unsigned int []);
+	 struct activity * [], unsigned int [], struct file_activity *);
 __printf_funct_t print_hdr_header
 	(void *, int, char *, struct file_magic *, struct file_header *,
-	 __nr_t, struct activity * [], unsigned int []);
+	 struct activity * [], unsigned int [], struct file_activity *);
 __printf_funct_t print_svg_header
 	(void *, int, char *, struct file_magic *, struct file_header *,
-	 __nr_t, struct activity * [], unsigned int []);
+	 struct activity * [], unsigned int [], struct file_activity *);
 
 #endif  /* _SADF_H */
