@@ -1,6 +1,6 @@
 /*
  * sadf_misc.c: Funtions used by sadf to display special records
- * (C) 2011-2017 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 2011-2018 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -23,7 +23,6 @@
 #include <string.h>
 
 #include "sadf.h"
-#include "sa.h"
 
 #ifdef USE_NLS
 #include <locale.h>
@@ -35,6 +34,8 @@
 
 extern unsigned int flags;
 extern char *seps[];
+extern unsigned int id_seq[];
+
 
 /*
  ***************************************************************************
@@ -46,11 +47,10 @@ extern char *seps[];
  * @utc		True if @cur_time is expressed in UTC.
  * @sep		Character used as separator.
  * @file_hdr	System activity file standard header.
- * @cpu_nr	CPU count associated with restart mark.
  ***************************************************************************
  */
 void print_dbppc_restart(char *cur_date, char *cur_time, int utc, char sep,
-			 struct file_header *file_hdr, unsigned int cpu_nr)
+			 struct file_header *file_hdr)
 {
 	printf("%s%c-1%c", file_hdr->sa_nodename, sep, sep);
 	if (strlen(cur_date)) {
@@ -61,7 +61,7 @@ void print_dbppc_restart(char *cur_date, char *cur_time, int utc, char sep,
 		printf(" UTC");
 	}
 	printf("%cLINUX-RESTART\t(%d CPU)\n",
-	       sep, cpu_nr > 1 ? cpu_nr - 1 : 1);
+	       sep, file_hdr->sa_cpu_nr > 1 ? file_hdr->sa_cpu_nr - 1 : 1);
 }
 
 /*
@@ -75,16 +75,14 @@ void print_dbppc_restart(char *cur_date, char *cur_time, int utc, char sep,
  * @cur_time	Time string of current restart message.
  * @utc		True if @cur_time is expressed in UTC.
  * @file_hdr	System activity file standard header.
- * @cpu_nr	CPU count associated with restart mark.
  ***************************************************************************
  */
 __printf_funct_t print_db_restart(int *tab, int action, char *cur_date,
-				  char *cur_time, int utc, struct file_header *file_hdr,
-				  unsigned int cpu_nr)
+				  char *cur_time, int utc, struct file_header *file_hdr)
 {
 	/* Actions F_BEGIN and F_END ignored */
 	if (action == F_MAIN) {
-		print_dbppc_restart(cur_date, cur_time, utc, ';', file_hdr, cpu_nr);
+		print_dbppc_restart(cur_date, cur_time, utc, ';', file_hdr);
 	}
 }
 
@@ -99,16 +97,14 @@ __printf_funct_t print_db_restart(int *tab, int action, char *cur_date,
  * @cur_time	Time string of current restart message.
  * @utc		True if @cur_time is expressed in UTC.
  * @file_hdr	System activity file standard header.
- * @cpu_nr	CPU count associated with restart mark.
  ***************************************************************************
  */
 __printf_funct_t print_ppc_restart(int *tab, int action, char *cur_date,
-				   char *cur_time, int utc, struct file_header *file_hdr,
-				   unsigned int cpu_nr)
+				   char *cur_time, int utc, struct file_header *file_hdr)
 {
 	/* Actions F_BEGIN and F_END ignored */
 	if (action == F_MAIN) {
-		print_dbppc_restart(cur_date, cur_time, utc, '\t', file_hdr, cpu_nr);
+		print_dbppc_restart(cur_date, cur_time, utc, '\t', file_hdr);
 	}
 }
 
@@ -122,23 +118,22 @@ __printf_funct_t print_ppc_restart(int *tab, int action, char *cur_date,
  * @cur_date	Date string of current restart message.
  * @cur_time	Time string of current restart message.
  * @utc		True if @cur_time is expressed in UTC.
- * @file_hdr	System activity file standard header (unused here).
- * @cpu_nr	CPU count associated with restart mark.
+ * @file_hdr	System activity file standard header.
  *
  * OUT:
  * @tab		Number of tabulations.
  ***************************************************************************
  */
 __printf_funct_t print_xml_restart(int *tab, int action, char *cur_date,
-				   char *cur_time, int utc, struct file_header *file_hdr,
-				   unsigned int cpu_nr)
+				   char *cur_time, int utc, struct file_header *file_hdr)
 {
 	if (action & F_BEGIN) {
 		xprintf((*tab)++, "<restarts>");
 	}
 	if (action & F_MAIN) {
 		xprintf(*tab, "<boot date=\"%s\" time=\"%s\" utc=\"%d\" cpu_count=\"%d\"/>",
-			cur_date, cur_time, utc ? 1 : 0, cpu_nr > 1 ? cpu_nr - 1 : 1);
+			cur_date, cur_time, utc ? 1 : 0,
+			file_hdr->sa_cpu_nr > 1 ? file_hdr->sa_cpu_nr - 1 : 1);
 	}
 	if (action & F_END) {
 		xprintf(--(*tab), "</restarts>");
@@ -155,16 +150,14 @@ __printf_funct_t print_xml_restart(int *tab, int action, char *cur_date,
  * @cur_date	Date string of current restart message.
  * @cur_time	Time string of current restart message.
  * @utc		True if @cur_time is expressed in UTC.
- * @file_hdr	System activity file standard header (unused here).
- * @cpu_nr	CPU count associated with restart mark.
+ * @file_hdr	System activity file standard header.
  *
  * OUT:
  * @tab		Number of tabulations.
  ***************************************************************************
  */
 __printf_funct_t print_json_restart(int *tab, int action, char *cur_date,
-				    char *cur_time, int utc, struct file_header *file_hdr,
-				    unsigned int cpu_nr)
+				    char *cur_time, int utc, struct file_header *file_hdr)
 {
 	static int sep = FALSE;
 
@@ -178,7 +171,8 @@ __printf_funct_t print_json_restart(int *tab, int action, char *cur_date,
 		}
 		xprintf((*tab)++, "{");
 		xprintf(*tab, "\"boot\": {\"date\": \"%s\", \"time\": \"%s\", \"utc\": %d, \"cpu_count\": %d}",
-			cur_date, cur_time, utc ? 1 : 0, cpu_nr > 1 ? cpu_nr - 1 : 1);
+			cur_date, cur_time, utc ? 1 : 0,
+			file_hdr->sa_cpu_nr > 1 ? file_hdr->sa_cpu_nr - 1 : 1);
 		xprintf0(--(*tab), "}");
 		sep = TRUE;
 	}
@@ -201,13 +195,11 @@ __printf_funct_t print_json_restart(int *tab, int action, char *cur_date,
  * @cur_date	Date string of current restart message.
  * @cur_time	Time string of current restart message.
  * @utc		True if @cur_time is expressed in UTC.
- * @file_hdr	System activity file standard header (unused here).
- * @cpu_nr	CPU count associated with restart mark.
+ * @file_hdr	System activity file standard header.
  ***************************************************************************
  */
 __printf_funct_t print_raw_restart(int *tab, int action, char *cur_date,
-				   char *cur_time, int utc, struct file_header *file_hdr,
-				   unsigned int cpu_nr)
+				   char *cur_time, int utc, struct file_header *file_hdr)
 {
 	/* Actions F_BEGIN and F_END ignored */
 	if (action == F_MAIN) {
@@ -215,7 +207,8 @@ __printf_funct_t print_raw_restart(int *tab, int action, char *cur_date,
 		if (strlen(cur_date) && utc) {
 			printf(" UTC");
 		}
-		printf("\tLINUX-RESTART\t(%d CPU)\n", cpu_nr > 1 ? cpu_nr - 1 : 1);
+		printf("; LINUX-RESTART (%d CPU)\n",
+		       file_hdr->sa_cpu_nr > 1 ? file_hdr->sa_cpu_nr - 1 : 1);
 	}
 }
 
@@ -401,7 +394,7 @@ __printf_funct_t print_raw_comment(int *tab, int action, char *cur_date,
 		if (strlen(cur_date) && utc) {
 			printf(" UTC");
 		}
-		printf("\tCOM %s\n", comment);
+		printf("; COM %s\n", comment);
 	}
 }
 
@@ -483,21 +476,21 @@ char *print_dbppc_timestamp(int fmt, struct file_header *file_hdr, char *cur_dat
 			    char *cur_time, int utc, unsigned long long itv)
 {
 	int isdb = (fmt == F_DB_OUTPUT);
-	static char pre[80];
-	char temp[80];
+	static char pre[512];
+	char temp1[128], temp2[256];
 
 	/* This substring appears on every output line, preformat it here */
-	snprintf(pre, 80, "%s%s%lld%s",
+	snprintf(temp1, sizeof(temp1), "%s%s%lld%s",
 		 file_hdr->sa_nodename, seps[isdb], itv, seps[isdb]);
 	if (strlen(cur_date)) {
-		snprintf(temp, 80, "%s%s ", pre, cur_date);
+		snprintf(temp2, sizeof(temp2), "%s%s ", temp1, cur_date);
 	}
 	else {
-		strcpy(temp, pre);
+		strcpy(temp2, temp1);
 	}
-	snprintf(pre, 80, "%s%s%s", temp, cur_time,
+	snprintf(pre, sizeof(pre), "%s%s%s", temp2, cur_time,
 		 strlen(cur_date) && utc ? " UTC" : "");
-	pre[79] = '\0';
+	pre[sizeof(pre) - 1] = '\0';
 
 	if (DISPLAY_HORIZONTALLY(flags)) {
 		printf("%s", pre);
@@ -688,9 +681,9 @@ __tm_funct_t print_raw_timestamp(void *parm, int action, char *cur_date,
  * @dfile	Name of system activity data file.
  * @file_magic	System activity file magic header.
  * @file_hdr	System activity file standard header.
- * @cpu_nr	Number of processors for current daily data file.
  * @act		Array of activities (unused here).
  * @id_seq	Activity sequence (unused here).
+ * @file_actlst	List of (known or unknown) activities in file (unused here).
  *
  * OUT:
  * @parm	Number of tabulations.
@@ -698,8 +691,9 @@ __tm_funct_t print_raw_timestamp(void *parm, int action, char *cur_date,
  */
 __printf_funct_t print_xml_header(void *parm, int action, char *dfile,
 				  struct file_magic *file_magic,
-				  struct file_header *file_hdr, __nr_t cpu_nr,
-				  struct activity *act[], unsigned int id_seq[])
+				  struct file_header *file_hdr,
+				  struct activity *act[], unsigned int id_seq[],
+				  struct file_activity *file_actlst)
 {
 	struct tm rectime, *loc_t;
 	char cur_time[TIMESTAMP_LEN];
@@ -726,7 +720,7 @@ __printf_funct_t print_xml_header(void *parm, int action, char *dfile,
 
 		xprintf(*tab, "<machine>%s</machine>", file_hdr->sa_machine);
 		xprintf(*tab, "<number-of-cpus>%d</number-of-cpus>",
-			cpu_nr > 1 ? cpu_nr - 1 : 1);
+			file_hdr->sa_cpu_nr > 1 ? file_hdr->sa_cpu_nr - 1 : 1);
 
 		/* Fill file timestmap structure (rectime) */
 		get_file_timestamp_struct(flags, &rectime, file_hdr);
@@ -755,9 +749,9 @@ __printf_funct_t print_xml_header(void *parm, int action, char *dfile,
  * @dfile	Name of system activity data file.
  * @file_magic	System activity file magic header.
  * @file_hdr	System activity file standard header.
- * @cpu_nr	Number of processors for current daily data file.
  * @act		Array of activities (unused here).
  * @id_seq	Activity sequence (unused here).
+ * @file_actlst	List of (known or unknown) activities in file (unused here).
  *
  * OUT:
  * @parm	Number of tabulations.
@@ -765,8 +759,9 @@ __printf_funct_t print_xml_header(void *parm, int action, char *dfile,
  */
 __printf_funct_t print_json_header(void *parm, int action, char *dfile,
 				   struct file_magic *file_magic,
-				   struct file_header *file_hdr, __nr_t cpu_nr,
-				   struct activity *act[], unsigned int id_seq[])
+				   struct file_header *file_hdr,
+				   struct activity *act[], unsigned int id_seq[],
+				   struct file_activity *file_actlst)
 {
 	struct tm rectime, *loc_t;
 	char cur_time[TIMESTAMP_LEN];
@@ -783,7 +778,7 @@ __printf_funct_t print_json_header(void *parm, int action, char *dfile,
 
 		xprintf(*tab, "\"machine\": \"%s\",", file_hdr->sa_machine);
 		xprintf(*tab, "\"number-of-cpus\": %d,",
-			cpu_nr > 1 ? cpu_nr - 1 : 1);
+			file_hdr->sa_cpu_nr > 1 ? file_hdr->sa_cpu_nr - 1 : 1);
 
 		/* Fill file timestmap structure (rectime) */
 		get_file_timestamp_struct(flags, &rectime, file_hdr);
@@ -815,18 +810,20 @@ __printf_funct_t print_json_header(void *parm, int action, char *dfile,
  * @dfile	Name of system activity data file.
  * @file_magic	System activity file magic header.
  * @file_hdr	System activity file standard header.
- * @cpu_nr	Number of processors for current daily data file.
  * @act		Array of activities.
  * @id_seq	Activity sequence.
+ * @file_actlst	List of (known or unknown) activities in file.
  ***************************************************************************
  */
 __printf_funct_t print_hdr_header(void *parm, int action, char *dfile,
 				  struct file_magic *file_magic,
-				  struct file_header *file_hdr, __nr_t cpu_nr,
-				  struct activity *act[], unsigned int id_seq[])
+				  struct file_header *file_hdr,
+				  struct activity *act[], unsigned int id_seq[],
+				  struct file_activity *file_actlst)
 {
 	int i, p;
 	struct tm rectime, *loc_t;
+	struct file_activity *fal;
 	char cur_time[TIMESTAMP_LEN];
 
 	/* Actions F_MAIN and F_END ignored */
@@ -848,12 +845,9 @@ __printf_funct_t print_hdr_header(void *parm, int action, char *dfile,
 		print_gal_header(localtime((const time_t *) &(file_hdr->sa_ust_time)),
 				 file_hdr->sa_sysname, file_hdr->sa_release,
 				 file_hdr->sa_nodename, file_hdr->sa_machine,
-				 cpu_nr > 1 ? cpu_nr - 1 : 1,
+				 file_hdr->sa_cpu_nr > 1 ? file_hdr->sa_cpu_nr - 1 : 1,
 				 PLAIN_OUTPUT);
 
-		printf(_("Number of CPU for last samples in file: %u\n"),
-		       file_hdr->sa_last_cpu_nr > 1 ? file_hdr->sa_last_cpu_nr - 1 : 1);
-
 		/* Fill file timestmap structure (rectime) */
 		get_file_timestamp_struct(flags, &rectime, file_hdr);
 		strftime(cur_time, sizeof(cur_time), "%Y-%m-%d", &rectime);
@@ -865,26 +859,37 @@ __printf_funct_t print_hdr_header(void *parm, int action, char *dfile,
 			printf("%s UTC\n", cur_time);
 		}
 
-		printf(_("Size of a long int: %d\n"), file_hdr->sa_sizeof_long);
+		/* File composition: file_header, file_activity, record_header */
+		printf(_("File composition: (%d,%d,%d),(%d,%d,%d),(%d,%d,%d)\n"),
+		       file_magic->hdr_types_nr[0], file_magic->hdr_types_nr[1], file_magic->hdr_types_nr[2],
+		       file_hdr->act_types_nr[0], file_hdr->act_types_nr[1], file_hdr->act_types_nr[2],
+		       file_hdr->rec_types_nr[0], file_hdr->rec_types_nr[1], file_hdr->rec_types_nr[2]);
 
-		/* Number of activities (number of volatile activities) in file */
-		printf("sa_act_nr (sa_vol_act_nr): %u (%u)\n",
-		       file_hdr->sa_act_nr, file_hdr->sa_vol_act_nr);
+		printf(_("Size of a long int: %d\n"), file_hdr->sa_sizeof_long);
+		printf("HZ = %lu\n", file_hdr->sa_hz);
+		printf(_("Number of activities in file: %u\n"),
+		       file_hdr->sa_act_nr);
 
 		printf(_("List of activities:\n"));
+		fal = file_actlst;
+		for (i = 0; i < file_hdr->sa_act_nr; i++, fal++) {
 
-		for (i = 0; i < NR_ACT; i++) {
-			if (!id_seq[i])
-				continue;
+			p = get_activity_position(act, fal->id, RESUME_IF_NOT_FOUND);
 
-			p = get_activity_position(act, id_seq[i], EXIT_IF_NOT_FOUND);
-
-			printf("%02d: %s\t(x%d)", act[p]->id, act[p]->name, act[p]->nr);
-			if (IS_MATRIX(act[p]->options)) {
-				printf("\t(x%d)", act[p]->nr2);
+			printf("%02d: [%02x] ", fal->id, fal->magic);
+			if (p >= 0) {
+				printf("%-20s", act[p]->name);
 			}
-			if (act[p]->magic == ACTIVITY_MAGIC_UNKNOWN) {
-				printf(_("\t[Unknown activity format]"));
+			else {
+				printf("%-20s", _("Unknown activity"));
+			}
+			printf(" %c:%4d", fal->has_nr ? 'Y' : 'N', fal->nr);
+			if (fal->nr2 > 1) {
+				printf("x%d", fal->nr2);
+			}
+			printf("\t(%d,%d,%d)", fal->types_nr[0], fal->types_nr[1], fal->types_nr[2]);
+			if ((p >= 0) && (act[p]->magic == ACTIVITY_MAGIC_UNKNOWN)) {
+				printf(_(" \t[Unknown format]"));
 			}
 			printf("\n");
 		}
@@ -903,54 +908,241 @@ __printf_funct_t print_hdr_header(void *parm, int action, char *dfile,
  * @dfile	Name of system activity data file (unused here).
  * @file_magic	System activity file magic header (unused here).
  * @file_hdr	System activity file standard header.
- * @cpu_nr	Number of processors for current daily data file.
  * @act		Array of activities (unused here).
  * @id_seq	Activity sequence (unused here).
+ * @file_actlst	List of (known or unknown) activities in file (unused here).
  ***************************************************************************
  */
 __printf_funct_t print_svg_header(void *parm, int action, char *dfile,
 				  struct file_magic *file_magic,
-				  struct file_header *file_hdr, __nr_t cpu_nr,
-				  struct activity *act[], unsigned int id_seq[])
+				  struct file_header *file_hdr,
+				  struct activity *act[], unsigned int id_seq[],
+				  struct file_activity *file_actlst)
 {
 	struct svg_hdr_parm *hdr_parm = (struct svg_hdr_parm *) parm;
+	unsigned int height = 0, ht = 0;
+	int i, p;
 
 	if (action & F_BEGIN) {
 		printf("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
 		printf("<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\" ");
 		printf("\"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n");
 		printf("<svg xmlns=\"http://www.w3.org/2000/svg\"");
+		if (DISPLAY_TOC(flags)) {
+			printf(" xmlns:xlink=\"http://www.w3.org/1999/xlink\"");
+		}
 		if (action & F_END) {
 			printf(">\n");
 		}
 	}
 
 	if (action & F_MAIN) {
+		if (SET_CANVAS_HEIGHT(flags)) {
+			/*
+			 * Option "-O height=..." used: @graph_nr is
+			 * the SVG canvas height set on the command line.
+			 */
+			height = hdr_parm->graph_nr;
+		}
+		else {
+			height = SVG_H_YSIZE +
+				 SVG_C_YSIZE * (DISPLAY_TOC(flags) ? hdr_parm->nr_act_dispd : 0) +
+				 SVG_T_YSIZE * hdr_parm->graph_nr;
+		}
+		if (height < 100) {
+			/* Min canvas height is 100 (at least to display "No data") */
+			height = 100;
+		}
 		printf(" width=\"%d\" height=\"%d\""
 		       " fill=\"black\" stroke=\"gray\" stroke-width=\"1\">\n",
-		       SVG_T_XSIZE * (hdr_parm->views_per_row),
-		       SET_CANVAS_HEIGHT(flags) ? hdr_parm->graph_nr
-						: SVG_H_YSIZE + SVG_T_YSIZE * (hdr_parm->graph_nr));
-		printf("<text x= \"0\" y=\"30\" text-anchor=\"start\" stroke=\"brown\">");
+		       SVG_T_XSIZE * (hdr_parm->views_per_row), height);
+		printf("<text x=\"0\" y=\"30\" text-anchor=\"start\" stroke=\"brown\">");
 		print_gal_header(localtime((const time_t *) &(file_hdr->sa_ust_time)),
 				 file_hdr->sa_sysname, file_hdr->sa_release,
 				 file_hdr->sa_nodename, file_hdr->sa_machine,
-				 cpu_nr > 1 ? cpu_nr - 1 : 1,
+				 file_hdr->sa_cpu_nr > 1 ? file_hdr->sa_cpu_nr - 1 : 1,
 				 PLAIN_OUTPUT);
 		printf("</text>\n");
+		if (DISPLAY_TOC(flags)) {
+			for (i = 0; i < NR_ACT; i++) {
+				if (!id_seq[i])
+					continue;	/* Activity not in file */
+
+				p = get_activity_position(act, id_seq[i], EXIT_IF_NOT_FOUND);
+				if (!IS_SELECTED(act[p]->options) || !act[p]->g_nr)
+					continue;	/* Activity not selected or no graph available */
+
+				printf("<a xlink:href=\"#g%d-0\" xlink:title=\"%s\">\n",
+				       act[p]->id, act[p]->name);
+				printf("<text x=\"10\" y=\"%d\">%s</text></a>\n",
+				       SVG_H_YSIZE + ht, act[p]->desc);
+				ht += SVG_C_YSIZE;
+			}
+		}
 	}
 
 	if (action & F_END) {
 		if (!(action & F_BEGIN)) {
 			if (!hdr_parm->graph_nr) {
 				/* No views displayed */
-				printf("<text x= \"0\" y=\"60\" text-anchor=\"start\" stroke=\"red\">");
+				printf("<text x= \"0\" y=\"%d\" text-anchor=\"start\" stroke=\"red\">",
+				       SVG_H_YSIZE +
+				       SVG_C_YSIZE * (DISPLAY_TOC(flags) ? hdr_parm->nr_act_dispd : 0));
 				printf("No data!</text>\n");
 			}
 			/* Give actual SVG height */
 			printf("<!-- Actual canvas height: %d -->\n",
-			       SVG_H_YSIZE + SVG_T_YSIZE * (hdr_parm->graph_nr));
+			       SVG_H_YSIZE +
+			       SVG_C_YSIZE * (DISPLAY_TOC(flags) ? hdr_parm->nr_act_dispd : 0) +
+			       SVG_T_YSIZE * hdr_parm->graph_nr);
 		}
 		printf("</svg>\n");
 	}
 }
+
+/*
+ ***************************************************************************
+ * Count the number of new network interfaces in current sample. If a new
+ * interface is found then add it to the linked list starting at
+ * @a->item_list.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @curr	Index in array for current sample statistics.
+ *
+ * RETURNS:
+ * Number of new interfaces identified in current sample that were not
+ * previously in the list.
+ ***************************************************************************
+ */
+__nr_t count_new_net_dev(struct activity *a, int curr)
+{
+	int i, nr = 0;
+	struct stats_net_dev *sndc;
+
+	for (i = 0; i < a->nr[curr]; i++) {
+		sndc = (struct stats_net_dev *) ((char *) a->buf[curr] + i * a->msize);
+
+		nr += add_list_item(&(a->item_list), sndc->interface, MAX_IFACE_LEN);
+	}
+
+	return nr;
+}
+
+/*
+ ***************************************************************************
+ * Count the number of new network interfaces in current sample. If a new
+ * interface is found then add it to the linked list starting at
+ * @a->item_list.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @curr	Index in array for current sample statistics.
+ *
+ * RETURNS:
+ * Number of new interfaces identified in current sample that were not
+ * previously in the list.
+ ***************************************************************************
+ */
+__nr_t count_new_net_edev(struct activity *a, int curr)
+{
+	int i, nr = 0;
+	struct stats_net_edev *snedc;
+
+	for (i = 0; i < a->nr[curr]; i++) {
+		snedc = (struct stats_net_edev *) ((char *) a->buf[curr] + i * a->msize);
+
+		nr += add_list_item(&(a->item_list), snedc->interface, MAX_IFACE_LEN);
+	}
+
+	return nr;
+}
+
+/*
+ ***************************************************************************
+ * Count the number of new filesystems in current sample. If a new
+ * filesystem is found then add it to the linked list starting at
+ * @a->item_list.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @curr	Index in array for current sample statistics.
+ *
+ * RETURNS:
+ * Number of new filesystems identified in current sample that were not
+ * previously in the list.
+ ***************************************************************************
+ */
+__nr_t count_new_filesystem(struct activity *a, int curr)
+{
+	int i, nr = 0;
+	struct stats_filesystem *sfc;
+
+	for (i = 0; i < a->nr[curr]; i++) {
+		sfc = (struct stats_filesystem *) ((char *) a->buf[curr] + i * a->msize);
+
+		nr += add_list_item(&(a->item_list), sfc->fs_name, MAX_FS_LEN);
+	}
+
+	return nr;
+}
+
+/*
+ ***************************************************************************
+ * Count the number of new fchosts in current sample. If a new
+ * fchost is found then add it to the linked list starting at
+ * @a->item_list.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @curr	Index in array for current sample statistics.
+ *
+ * RETURNS:
+ * Number of new fchosts identified in current sample that were not
+ * previously in the list.
+ ***************************************************************************
+ */
+__nr_t count_new_fchost(struct activity *a, int curr)
+{
+	int i, nr = 0;
+	struct stats_fchost *sfcc;
+
+	for (i = 0; i < a->nr[curr]; i++) {
+		sfcc = (struct stats_fchost *) ((char *) a->buf[curr] + i * a->msize);
+
+		nr += add_list_item(&(a->item_list), sfcc->fchost_name, MAX_FCH_LEN);
+	}
+
+	return nr;
+}
+
+/*
+ ***************************************************************************
+ * Count the number of new block devices in current sample. If a new
+ * block device is found then add it to the linked list starting at
+ * @a->item_list.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @curr	Index in array for current sample statistics.
+ *
+ * RETURNS:
+ * Number of new block devices identified in current sample that were not
+ * previously in the list.
+ ***************************************************************************
+ */
+__nr_t count_new_disk(struct activity *a, int curr)
+{
+	int i, nr = 0;
+	struct stats_disk *sdc;
+
+	for (i = 0; i < a->nr[curr]; i++) {
+		sdc = (struct stats_disk *) ((char *) a->buf[curr] + i * a->msize);
+
+		nr += add_list_item(&(a->item_list),
+				    get_sa_devname(sdc->major, sdc->minor, flags),
+				    MAX_DEV_LEN);
+	}
+
+	return nr;
+}
