@@ -48,6 +48,7 @@
 #include "version.h"
 #include "tapestat.h"
 #include "common.h"
+#include "rd_stats.h"
 #include "count.h"
 
 #ifdef USE_NLS
@@ -71,6 +72,9 @@ char timestamp[TIMESTAMP_LEN];
 
 struct sigaction alrm_act;
 
+/* Number of decimal places */
+int dplaces_nr = -1;
+
 /*
  * For tape stats - it would be extremely rare for there to be a very large
  * number of tape drives attached to a system. I wouldn't expect to see more
@@ -611,9 +615,6 @@ int main(int argc, char **argv)
 	/* Init color strings */
 	init_colors();
 
-	/* Get HZ */
-	get_HZ();
-
 	/* Process args... */
 	while (opt < argc) {
 
